@@ -204,3 +204,32 @@ VLAN_MODE_TRUNK = 2
 HOST_SHUTDOWN_ACTION_TURN_OFF = 2
 HOST_SHUTDOWN_ACTION_SAVE = 3
 HOST_SHUTDOWN_ACTION_SHUTDOWN = 4
+
+# VM snapshot types
+VM_SNAPSHOT_TYPE_DISABLED = 2
+VM_SNAPSHOT_TYPE_PROD_FALLBACK = 3
+VM_SNAPSHOT_TYPE_PROD_ENFORCED = 4
+VM_SNAPSHOT_TYPE_STANDARD = 5
+
+VM_VERSION_5_0 = '5.0'
+VM_VERSION_6_2 = '6.2'
+VM_VERSION_8_0 = '8.0'
+VM_VERSION_254_0 = '254.0'
+
+DEFAULT_WMI_EVENT_TIMEOUT_MS = 2000
+
+SCSI_UID_SCSI_NAME_STRING = 8
+SCSI_UID_FCPH_NAME = 3
+SCSI_UID_EUI64 = 2
+SCSI_UID_VENDOR_ID = 1
+SCSI_UID_VENDOR_SPECIFIC = 0
+
+# The following SCSI Unique ID formats are accepted by Windows, in this
+# specific order of precedence.
+SUPPORTED_SCSI_UID_FORMATS = [
+    SCSI_UID_SCSI_NAME_STRING,
+    SCSI_UID_FCPH_NAME,
+    SCSI_UID_EUI64,
+    SCSI_UID_VENDOR_ID,
+    SCSI_UID_VENDOR_SPECIFIC
+]
