@@ -14,7 +14,6 @@
 #    under the License.
 
 import ddt
-
 import mock
 from six.moves import range  # noqa
 
@@ -74,6 +73,7 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         self._vmutils._conn_attr = mock.MagicMock()
         self._vmutils._jobutils = mock.MagicMock()
         self._vmutils._pathutils = mock.MagicMock()
+        self._jobutils = self._vmutils._jobutils
 
     def test_get_vm_summary_info(self):
         self._lookup_vm()
@@ -128,6 +128,13 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         vssd = self._vmutils._lookup_vm_check(self._FAKE_VM_NAME)
         self.assertEqual(expected_vssd, vssd)
 
+    @mock.patch.object(vmutils.VMUtils, '_lookup_vm')
+    def test_vm_exists(self, mock_lookup_vm):
+        result = self._vmutils.vm_exists(mock.sentinel.vm_name)
+
+        self.assertTrue(result)
+        mock_lookup_vm.assert_called_once_with(mock.sentinel.vm_name, False)
+
     def test_set_vm_memory_static(self):
         self._test_set_vm_memory_dynamic(dynamic_memory_ratio=1.0)
 
@@ -302,9 +309,12 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
     @mock.patch.object(vmutils.VMUtils, '_modify_virtual_system')
     @mock.patch.object(vmutils.VMUtils, '_set_vm_vcpus')
     @mock.patch.object(vmutils.VMUtils, '_set_vm_memory')
+    @mock.patch.object(vmutils.VMUtils, '_set_vm_snapshot_type')
     @mock.patch.object(vmutils.VMUtils, '_lookup_vm_check')
-    def test_update_vm(self, mock_lookup_vm_check, mock_set_mem,
-                       mock_set_vcpus, mock_modify_virtual_system,
+    def test_update_vm(self, mock_lookup_vm_check,
+                       mock_set_vm_snap_type,
+                       mock_set_mem, mock_set_vcpus,
+                       mock_modify_virtual_system,
                        host_shutdown_action=None,
                        configuration_root_dir=None, vnuma_enabled=None):
         mock_vmsettings = mock_lookup_vm_check.return_value
@@ -314,9 +324,11 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
             mock.sentinel.vcpus_per_numa, mock.sentinel.limit_cpu_features,
             mock.sentinel.dynamic_mem_ratio, configuration_root_dir,
             host_shutdown_action=host_shutdown_action,
-            vnuma_enabled=vnuma_enabled)
+            vnuma_enabled=vnuma_enabled,
+            snapshot_type=mock.sentinel.snap_type)
 
-        mock_lookup_vm_check.assert_called_once_with(mock.sentinel.vm_name)
+        mock_lookup_vm_check.assert_called_once_with(mock.sentinel.vm_name,
+                                                     for_update=True)
         mock_set_mem.assert_called_once_with(
             mock_vmsettings, mock.sentinel.memory_mb,
             mock.sentinel.memory_per_numa, mock.sentinel.dynamic_mem_ratio)
@@ -341,11 +353,11 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         if vnuma_enabled:
             self.assertEqual(vnuma_enabled, mock_vmsettings.VirtualNumaEnabled)
 
-        if configuration_root_dir or host_shutdown_action or vnuma_enabled:
-            mock_modify_virtual_system.assert_called_once_with(
-                mock_vmsettings)
-        else:
-            mock_modify_virtual_system.assert_not_called()
+        mock_set_vm_snap_type.assert_called_once_with(
+            mock_vmsettings, mock.sentinel.snap_type)
+
+        mock_modify_virtual_system.assert_called_once_with(
+            mock_vmsettings)
 
     @mock.patch.object(vmutils.VMUtils, '_set_vm_memory')
     @mock.patch.object(vmutils.VMUtils, '_create_vm_obj')
@@ -489,6 +501,139 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         mock_get_ide_ctrl.assert_called_with(mock_vm, self._FAKE_CTRL_ADDR)
         self.assertTrue(mock_get_new_rsd.called)
 
+    @ddt.data(constants.DISK, constants.DVD)
+    @mock.patch.object(vmutils.VMUtils, '_get_new_resource_setting_data')
+    def test_attach_drive(self, drive_type, mock_get_new_rsd):
+        mock_vm = self._lookup_vm()
+
+        mock_drive_res = mock.Mock()
+        mock_disk_res = mock.Mock()
+
+        mock_get_new_rsd.side_effect = [mock_drive_res, mock_disk_res]
+        self._jobutils.add_virt_resource.side_effect = [
+            [mock.sentinel.drive_res_path],
+            [mock.sentinel.disk_res_path]]
+
+        self._vmutils.attach_drive(mock.sentinel.vm_name,
+                                   mock.sentinel.disk_path,
+                                   mock.sentinel.ctrl_path,
+                                   mock.sentinel.drive_addr,
+                                   drive_type)
+
+        self._vmutils._lookup_vm_check.assert_called_once_with(
+            mock.sentinel.vm_name, as_vssd=False)
+
+        if drive_type == constants.DISK:
+            exp_res_sub_types = [self._vmutils._DISK_DRIVE_RES_SUB_TYPE,
+                                 self._vmutils._HARD_DISK_RES_SUB_TYPE]
+        else:
+            exp_res_sub_types = [self._vmutils._DVD_DRIVE_RES_SUB_TYPE,
+                                 self._vmutils._DVD_DISK_RES_SUB_TYPE]
+
+        mock_get_new_rsd.assert_has_calls(
+            [mock.call(exp_res_sub_types[0]),
+             mock.call(exp_res_sub_types[1],
+                       self._vmutils._STORAGE_ALLOC_SETTING_DATA_CLASS)])
+
+        self.assertEqual(mock.sentinel.ctrl_path, mock_drive_res.Parent)
+        self.assertEqual(mock.sentinel.drive_addr, mock_drive_res.Address)
+        self.assertEqual(mock.sentinel.drive_addr,
+                         mock_drive_res.AddressOnParent)
+
+        self.assertEqual(mock.sentinel.drive_res_path,
+                         mock_disk_res.Parent)
+        self.assertEqual([mock.sentinel.disk_path],
+                         mock_disk_res.HostResource)
+
+        self._jobutils.add_virt_resource.assert_has_calls(
+            [mock.call(mock_drive_res, mock_vm),
+             mock.call(mock_disk_res, mock_vm)])
+
+    @mock.patch.object(vmutils.VMUtils, '_get_wmi_obj')
+    @mock.patch.object(vmutils.VMUtils, '_get_new_resource_setting_data')
+    def test_attach_drive_exc(self, mock_get_new_rsd, mock_get_wmi_obj):
+        self._lookup_vm()
+
+        mock_drive_res = mock.Mock()
+        mock_disk_res = mock.Mock()
+
+        mock_get_new_rsd.side_effect = [mock_drive_res, mock_disk_res]
+        self._jobutils.add_virt_resource.side_effect = [
+            [mock.sentinel.drive_res_path],
+            exceptions.OSWinException]
+        mock_get_wmi_obj.return_value = mock.sentinel.attached_drive_res
+
+        self.assertRaises(exceptions.OSWinException,
+                          self._vmutils.attach_drive,
+                          mock.sentinel.vm_name,
+                          mock.sentinel.disk_path,
+                          mock.sentinel.ctrl_path,
+                          mock.sentinel.drive_addr,
+                          constants.DISK)
+
+        mock_get_wmi_obj.assert_called_once_with(mock.sentinel.drive_res_path)
+        self._jobutils.remove_virt_resource.assert_called_once_with(
+            mock.sentinel.attached_drive_res)
+
+    @mock.patch.object(vmutils.VMUtils,
+                       '_get_mounted_disk_resource_from_path')
+    def test_get_disk_attachment_info_detached(self, mock_get_disk_res):
+        mock_get_disk_res.return_value = None
+        self.assertRaises(exceptions.DiskNotFound,
+                          self._vmutils.get_disk_attachment_info,
+                          mock.sentinel.disk_path,
+                          mock.sentinel.is_physical)
+
+        mock_get_disk_res.assert_called_once_with(
+            mock.sentinel.disk_path,
+            mock.sentinel.is_physical)
+
+    @ddt.data(True, False)
+    @mock.patch.object(vmutils.VMUtils,
+                       '_get_mounted_disk_resource_from_path')
+    @mock.patch.object(vmutils.VMUtils,
+                       '_get_disk_controller_type')
+    @mock.patch.object(vmutils.VMUtils,
+                       '_get_wmi_obj')
+    def test_get_disk_attachment_info(self, is_physical,
+                                      mock_get_wmi_obj,
+                                      mock_get_disk_ctrl_type,
+                                      mock_get_disk_res):
+        mock_res = mock_get_disk_res.return_value
+        exp_res = mock_res if is_physical else mock_get_wmi_obj.return_value
+
+        fake_slot = 5
+        exp_res.AddressOnParent = str(fake_slot)
+
+        exp_att_info = dict(
+            controller_slot=fake_slot,
+            controller_path=exp_res.Parent,
+            controller_type=mock_get_disk_ctrl_type.return_value)
+
+        att_info = self._vmutils.get_disk_attachment_info(
+            mock.sentinel.disk_path,
+            is_physical)
+        self.assertEqual(exp_att_info, att_info)
+
+        if not is_physical:
+            mock_get_wmi_obj.assert_called_once_with(mock_res.Parent)
+        mock_get_disk_ctrl_type.assert_called_once_with(exp_res.Parent)
+
+    @ddt.data(vmutils.VMUtils._SCSI_CTRL_RES_SUB_TYPE,
+              vmutils.VMUtils._IDE_CTRL_RES_SUB_TYPE)
+    @mock.patch.object(vmutils.VMUtils, '_get_wmi_obj')
+    def test_get_disk_controller_type(self, res_sub_type, mock_get_wmi_obj):
+        mock_ctrl = mock_get_wmi_obj.return_value
+        mock_ctrl.ResourceSubType = res_sub_type
+
+        exp_ctrl_type = self._vmutils._disk_ctrl_type_mapping[res_sub_type]
+
+        ctrl_type = self._vmutils._get_disk_controller_type(
+            mock.sentinel.ctrl_path)
+        self.assertEqual(exp_ctrl_type, ctrl_type)
+
+        mock_get_wmi_obj.assert_called_once_with(mock.sentinel.ctrl_path)
+
     @mock.patch.object(vmutils.VMUtils, '_get_new_resource_setting_data')
     def test_create_scsi_controller(self, mock_get_new_rsd):
         mock_vm = self._lookup_vm()
@@ -624,20 +769,24 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
             mock.sentinel.fake_new_mounted_disk_path,
             mock_rasds[0].HostResource[0])
 
-    def test_take_vm_snapshot(self):
-        mock_vm = self._lookup_vm()
-        mock_snap = mock.MagicMock(Antecedent=mock_vm)
-        self._vmutils._conn.Msvm_MostCurrentSnapshotInBranch.return_value = [
-            mock.MagicMock(), mock_snap]
+    @mock.patch.object(vmutils.VMUtils, '_modify_virtual_system')
+    @ddt.data(None, mock.sentinel.snap_name)
+    def test_take_vm_snapshot(self, snap_name, mock_modify_virtual_system):
+        self._lookup_vm()
+        mock_snap = mock.Mock(ElementName=mock.sentinel.default_snap_name)
 
         mock_svc = self._get_snapshot_service()
         mock_svc.CreateSnapshot.return_value = (self._FAKE_JOB_PATH,
                                                 mock.MagicMock(),
                                                 self._FAKE_RET_VAL)
 
-        snap_path = self._vmutils.take_vm_snapshot(self._FAKE_VM_NAME)
+        mock_job = self._vmutils._jobutils.check_ret_val.return_value
+        mock_job.associators.return_value = [mock_snap]
 
-        self.assertEqual(mock_snap.Dependent.path_.return_value, snap_path)
+        snap_path = self._vmutils.take_vm_snapshot(self._FAKE_VM_NAME,
+                                                   snap_name)
+
+        self.assertEqual(mock_snap.path_.return_value, snap_path)
         mock_svc.CreateSnapshot.assert_called_with(
             AffectedSystem=self._FAKE_VM_PATH,
             SnapshotType=self._vmutils._SNAPSHOT_FULL)
@@ -645,6 +794,41 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         self._vmutils._jobutils.check_ret_val.assert_called_once_with(
             self._FAKE_RET_VAL, self._FAKE_JOB_PATH)
 
+        mock_job.associators.assert_called_once_with(
+            wmi_result_class=self._vmutils._VIRTUAL_SYSTEM_SETTING_DATA_CLASS,
+            wmi_association_class=self._vmutils._AFFECTED_JOB_ELEMENT_CLASS)
+
+        if snap_name:
+            self.assertEqual(snap_name, mock_snap.ElementName)
+            mock_modify_virtual_system.assert_called_once_with(mock_snap)
+        else:
+            self.assertEqual(mock.sentinel.default_snap_name,
+                             mock_snap.ElementName)
+            mock_modify_virtual_system.assert_not_called()
+
+    @ddt.data(None, mock.sentinel.snap1)
+    def test_get_vm_snapshots(self, snap_name):
+        mock_snap1 = mock.Mock(ElementName=mock.sentinel.snap1)
+        mock_snap2 = mock.Mock(ElementName=mock.sentinel.snap2)
+
+        mock_vm = self._lookup_vm()
+        mock_vm.associators.return_value = [mock_snap1, mock_snap2]
+
+        snaps = self._vmutils.get_vm_snapshots(mock.sentinel.vm_name,
+                                               snap_name)
+
+        expected_snaps = [mock_snap1.path_.return_value]
+        if not snap_name:
+            expected_snaps += [mock_snap2.path_.return_value]
+
+        self.assertEqual(expected_snaps, snaps)
+
+        mock_vm.associators.assert_called_once_with(
+            wmi_association_class=(
+                self._vmutils._VIRTUAL_SYSTEM_SNAP_ASSOC_CLASS),
+            wmi_result_class=(
+                self._vmutils._VIRTUAL_SYSTEM_SETTING_DATA_CLASS))
+
     def test_remove_vm_snapshot(self):
         mock_svc = self._get_snapshot_service()
         getattr(mock_svc, self._DESTROY_SNAPSHOT).return_value = (
@@ -681,20 +865,26 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
         mock_disk = self._prepare_mock_disk()
 
         self._vmutils.detach_vm_disk(self._FAKE_VM_NAME,
-                                     self._FAKE_HOST_RESOURCE)
+                                     self._FAKE_HOST_RESOURCE,
+                                     serial=mock.sentinel.serial)
         self._vmutils._jobutils.remove_virt_resource.assert_called_once_with(
             mock_disk)
 
-    def test_get_mounted_disk_resource_from_path(self):
-        mock_disk_1 = mock.MagicMock()
-        mock_disk_2 = mock.MagicMock()
-        mock_disk_2.HostResource = [self._FAKE_MOUNTED_DISK_PATH]
-        self._vmutils._conn.query.return_value = [mock_disk_1, mock_disk_2]
+    @ddt.data(None, mock.sentinel.serial)
+    def test_get_mounted_disk_resource_from_path(self, serial):
+        mock_disk = mock.MagicMock()
+
+        if serial:
+            self._vmutils._conn.query.return_value = [mock_disk]
+        else:
+            mock_disk.HostResource = [self._FAKE_MOUNTED_DISK_PATH]
+            self._vmutils._conn.query.return_value = [
+                mock.MagicMock(), mock_disk]
 
         physical_disk = self._vmutils._get_mounted_disk_resource_from_path(
-            self._FAKE_MOUNTED_DISK_PATH, True)
+            self._FAKE_MOUNTED_DISK_PATH, True, serial=serial)
 
-        self.assertEqual(mock_disk_2, physical_disk)
+        self.assertEqual(mock_disk, physical_disk)
 
     def test_get_controller_volume_paths(self):
         self._prepare_mock_disk()
@@ -1023,9 +1213,9 @@ class VMUtilsTestCase(test_base.OsWinBaseTestCase):
                                               enabled_state)
         mock_tpool.execute.assert_has_calls(
             fake_listener,
-            [mock.call(self._vmutils._DEFAULT_EVENT_TIMEOUT_MS)] * 4)
+            [mock.call(constants.DEFAULT_WMI_EVENT_TIMEOUT_MS)] * 4)
         mock_sleep.assert_called_once_with(
-            self._vmutils._DEFAULT_EVENT_TIMEOUT_MS / 1000)
+            constants.DEFAULT_WMI_EVENT_TIMEOUT_MS / 1000)
 
     def _test_get_vm_generation(self, vm_gen):
         mock_settings = self._lookup_vm()
