@@ -23,15 +23,32 @@ import re
 
 from eventlet import patcher
 from eventlet import tpool
+from oslo_log import log as logging
 from oslo_utils import units
+import six
 
-from os_win._i18n import _, _LE
+from os_win._i18n import _
+from os_win import conf
 from os_win import constants
 from os_win import exceptions
 from os_win.utils import _wqlutils
 from os_win.utils import baseutils
 from os_win.utils import jobutils
 
+CONF = conf.CONF
+LOG = logging.getLogger(__name__)
+
+_PORT_PROFILE_ATTR_MAP = {
+    "profile_id": "ProfileId",
+    "profile_data": "ProfileData",
+    "profile_name": "ProfileName",
+    "net_cfg_instance_id": "NetCfgInstanceId",
+    "cdn_label_id": "CdnLabelId",
+    "cdn_label_string": "CdnLabelString",
+    "vendor_id": "VendorId",
+    "vendor_name": "VendorName",
+}
+
 
 class NetworkUtils(baseutils.BaseUtilsVirt):
 
@@ -43,6 +60,7 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
     _ETHERNET_SWITCH_PORT = 'Msvm_EthernetSwitchPort'
     _PORT_ALLOC_SET_DATA = 'Msvm_EthernetPortAllocationSettingData'
     _PORT_VLAN_SET_DATA = 'Msvm_EthernetSwitchPortVlanSettingData'
+    _PORT_PROFILE_SET_DATA = 'Msvm_EthernetSwitchPortProfileSettingData'
     _PORT_SECURITY_SET_DATA = 'Msvm_EthernetSwitchPortSecuritySettingData'
     _PORT_ALLOC_ACL_SET_DATA = 'Msvm_EthernetSwitchPortAclSettingData'
     _PORT_BANDWIDTH_SET_DATA = 'Msvm_EthernetSwitchPortBandwidthSettingData'
@@ -84,6 +102,7 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
     _switches = {}
     _switch_ports = {}
     _vlan_sds = {}
+    _profile_sds = {}
     _vsid_sds = {}
     _sg_acl_sds = {}
     _bandwidth_sds = {}
@@ -91,8 +110,13 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
     def __init__(self):
         super(NetworkUtils, self).__init__()
         self._jobutils = jobutils.JobUtils()
+        self._enable_cache = CONF.os_win.cache_temporary_wmi_objects
 
     def init_caches(self):
+        if not self._enable_cache:
+            LOG.info('WMI caching is disabled.')
+            return
+
         for vswitch in self._conn.Msvm_VirtualEthernetSwitch():
             self._switches[vswitch.ElementName] = vswitch
 
@@ -106,6 +130,13 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
             "Microsoft:[0-9A-F-]*\\\\[0-9A-F-]*\\\\[0-9A-F-]",
             flags=re.IGNORECASE)
 
+        # map between switch port's InstanceID and their Port Profile settings
+        # data WMI objects.
+        for profile in self._conn.Msvm_EthernetSwitchPortProfileSettingData():
+            match = switch_port_id_regex.match(profile.InstanceID)
+            if match:
+                self._profile_sds[match.group()] = profile
+
         # map between switch port's InstanceID and their VLAN setting data WMI
         # objects.
         for vlan_sd in self._conn.Msvm_EthernetSwitchPortVlanSettingData():
@@ -129,10 +160,13 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
                 self._bandwidth_sds[match.group()] = bandwidth_sd
 
     def update_cache(self):
+        if not self._enable_cache:
+            return
+
         # map between switch port ID and switch port WMI object.
-        self._switch_ports = {
-            port.ElementName: port for port in
-            self._conn.Msvm_EthernetPortAllocationSettingData()}
+        self._switch_ports.clear()
+        for port in self._conn.Msvm_EthernetPortAllocationSettingData():
+            self._switch_ports[port.ElementName] = port
 
     def clear_port_sg_acls_cache(self, switch_port_name):
         self._sg_acl_sds.pop(switch_port_name, None)
@@ -155,8 +189,8 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
         if not len(vswitch):
             raise exceptions.HyperVException(_('VSwitch not found: %s') %
                                              vswitch_name)
-
-        self._switches[vswitch_name] = vswitch[0]
+        if self._enable_cache:
+            self._switches[vswitch_name] = vswitch[0]
         return vswitch[0]
 
     def _get_vswitch_external_port(self, vswitch_name):
@@ -212,8 +246,7 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
         vnic_settings = self._conn.Msvm_SyntheticEthernetPortSettingData(
             ElementName=vnic_name)
         if not vnic_settings:
-            raise exceptions.HyperVException(
-                message=_('Vnic not found: %s') % vnic_name)
+            raise exceptions.HyperVvNicNotFound(vnic_name=vnic_name)
         return vnic_settings[0]
 
     def get_vnic_event_listener(self, event_type):
@@ -307,10 +340,51 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
                 pass
 
         self._switch_ports.pop(switch_port_name, None)
+        self._profile_sds.pop(sw_port.InstanceID, None)
         self._vlan_sds.pop(sw_port.InstanceID, None)
         self._vsid_sds.pop(sw_port.InstanceID, None)
         self._bandwidth_sds.pop(sw_port.InstanceID, None)
 
+    def set_vswitch_port_profile_id(self, switch_port_name, profile_id,
+                                    profile_data, profile_name, vendor_name,
+                                    **kwargs):
+        """Sets up the port profile id.
+
+        :param switch_port_name: The ElementName of the vSwitch port.
+        :param profile_id: The profile id to be set for the given switch port.
+        :param profile_data: Additional data for the Port Profile.
+        :param profile_name: The name of the Port Profile.
+        :param net_cfg_instance_id: Unique device identifier of the
+            sub-interface.
+        :param cdn_label_id: The CDN Label Id.
+        :param cdn_label_string: The CDN label string.
+        :param vendor_id: The id of the Vendor defining the profile.
+        :param vendor_name: The name of the Vendor defining the profile.
+        """
+        port_alloc = self._get_switch_port_allocation(switch_port_name)[0]
+        port_profile = self._get_profile_setting_data_from_port_alloc(
+            port_alloc)
+
+        new_port_profile = self._prepare_profile_sd(
+            profile_id=profile_id, profile_data=profile_data,
+            profile_name=profile_name, vendor_name=vendor_name, **kwargs)
+
+        if port_profile:
+            # Removing the feature because it cannot be modified
+            # due to a wmi exception.
+            self._jobutils.remove_virt_feature(port_profile)
+
+            # remove from cache.
+            self._profile_sds.pop(port_alloc.InstanceID, None)
+
+        try:
+            self._jobutils.add_virt_feature(new_port_profile, port_alloc)
+        except Exception as ex:
+            raise exceptions.HyperVException(
+                'Unable to set port profile settings %(port_profile)s '
+                'for port %(port)s. Error: %(error)s' %
+                dict(port_profile=new_port_profile, port=port_alloc, error=ex))
+
     def set_vswitch_port_vlan_id(self, vlan_id=None, switch_port_name=None,
                                  **kwargs):
         """Sets up operation mode, VLAN ID and VLAN trunk for the given port.
@@ -374,6 +448,21 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
             raise exceptions.HyperVException(
                 _('Port VLAN not found: %s') % switch_port_name)
 
+    def _prepare_profile_sd(self, **kwargs):
+        profile_id_settings = self._create_default_setting_data(
+            self._PORT_PROFILE_SET_DATA)
+
+        for argument_name, attr_name in _PORT_PROFILE_ATTR_MAP.items():
+            attribute = kwargs.pop(argument_name, None)
+            if attribute is None:
+                continue
+            setattr(profile_id_settings, attr_name, attribute)
+
+        if kwargs:
+            raise TypeError("Unrecognized attributes %r" % kwargs)
+
+        return profile_id_settings
+
     def _prepare_vlan_sd_access_mode(self, vlan_settings, vlan_id):
         if vlan_settings:
             # the given vlan_id might be None.
@@ -455,6 +544,10 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
             raise exceptions.HyperVException(
                 _('Port Security Settings not found: %s') % switch_port_name)
 
+    def _get_profile_setting_data_from_port_alloc(self, port_alloc):
+        return self._get_setting_data_from_port_alloc(
+            port_alloc, self._profile_sds, self._PORT_PROFILE_SET_DATA)
+
     def _get_vlan_setting_data_from_port_alloc(self, port_alloc):
         return self._get_setting_data_from_port_alloc(
             port_alloc, self._vlan_sds, self._PORT_VLAN_SET_DATA)
@@ -475,7 +568,7 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
             _wqlutils.get_element_associated_class(
                 self._conn, data_class,
                 element_instance_id=port_alloc.InstanceID))
-        if setting_data:
+        if setting_data and self._enable_cache:
             cache[port_alloc.InstanceID] = setting_data
         return setting_data
 
@@ -492,7 +585,8 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
             # newly created setting data cannot be cached, they do not
             # represent real objects yet.
             # if it was found, it means that it was not created.
-            self._switch_ports[switch_port_name] = switch_port
+            if self._enable_cache:
+                self._switch_ports[switch_port_name] = switch_port
         elif expected:
             raise exceptions.HyperVPortNotFoundException(
                 port_name=switch_port_name)
@@ -652,7 +746,8 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
         acls = _wqlutils.get_element_associated_class(
             self._conn, self._PORT_EXT_ACL_SET_DATA,
             element_instance_id=port.InstanceID)
-        self._sg_acl_sds[port.ElementName] = acls
+        if self._enable_cache:
+            self._sg_acl_sds[port.ElementName] = acls
 
         return acls
 
@@ -665,7 +760,7 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
         return acl
 
     def _create_security_acl(self, sg_rule, weight):
-        # Acl instance can be created new eachtime, the object should be
+        # Acl instance can be created new each time, the object should be
         # of type ExtendedEthernetSettingsData.
         acl = self._create_default_setting_data(self._PORT_EXT_ACL_SET_DATA)
         acl.set(**sg_rule.to_dict())
@@ -745,12 +840,12 @@ class NetworkUtils(baseutils.BaseUtilsVirt):
         try:
             self._jobutils.add_virt_feature(bandwidth, port_alloc)
         except Exception as ex:
-            if '0x80070057' in ex.message:
+            if '0x80070057' in six.text_type(ex):
                 raise exceptions.InvalidParameterValue(
                     param_name="qos_rule", param_value=qos_rule)
             raise exceptions.HyperVException(
-                _LE('Unable to set qos rule %(qos_rule)s for port %(port)s. '
-                    'Error: %(error)s') %
+                'Unable to set qos rule %(qos_rule)s for port %(port)s. '
+                'Error: %(error)s' %
                 dict(qos_rule=qos_rule, port=port_alloc, error=ex))
 
     def remove_port_qos_rule(self, port_id):
