@@ -15,23 +15,18 @@
 #    under the License.
 
 import ctypes
-import sys
 
 from oslo_log import log as logging
 
-from os_win._i18n import _LE
 from os_win import _utils
 from os_win import exceptions
+from os_win.utils.winapi import constants as w_const
+from os_win.utils.winapi import libs as w_lib
 
-if sys.platform == 'win32':
-    kernel32 = ctypes.windll.kernel32
+kernel32 = w_lib.get_shared_lib_handle(w_lib.KERNEL32)
 
 LOG = logging.getLogger(__name__)
 
-FORMAT_MESSAGE_FROM_SYSTEM = 0x00001000
-FORMAT_MESSAGE_ALLOCATE_BUFFER = 0x00000100
-FORMAT_MESSAGE_IGNORE_INSERTS = 0x00000200
-
 
 class Win32Utils(object):
     def __init__(self):
@@ -100,8 +95,9 @@ class Win32Utils(object):
         message_buffer = ctypes.c_char_p()
 
         kernel32.FormatMessageA(
-            FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_ALLOCATE_BUFFER |
-            FORMAT_MESSAGE_IGNORE_INSERTS,
+            w_const.FORMAT_MESSAGE_FROM_SYSTEM |
+            w_const.FORMAT_MESSAGE_ALLOCATE_BUFFER |
+            w_const.FORMAT_MESSAGE_IGNORE_INSERTS,
             None, error_code, 0, ctypes.byref(message_buffer), 0, None)
 
         error_message = message_buffer.value
@@ -119,20 +115,16 @@ class Win32Utils(object):
         return hresult & 0xFFFF
 
     def get_com_err_code(self, com_error):
-        hres = self.get_com_error_hresult(com_error)
+        hres = _utils.get_com_error_hresult(com_error)
         if hres is not None:
             return self.hresult_to_err_code(hres)
 
-    @staticmethod
-    def get_com_error_hresult(com_error):
-        try:
-            return ctypes.c_uint(com_error.excepinfo[5]).value
-        except Exception:
-            LOG.debug("Unable to retrieve COM error hresult: %s", com_error)
-
     def local_free(self, handle):
         try:
             self._run_and_check_output(kernel32.LocalFree, handle)
         except exceptions.Win32Exception:
-            LOG.exception(_LE("Could not deallocate memory. "
-                              "There could be a memory leak."))
+            LOG.exception("Could not deallocate memory. "
+                          "There could be a memory leak.")
+
+    def close_handle(self, handle):
+        kernel32.CloseHandle(handle)
